#include <iostream.h>
#include "alarm.hh"

#ifdef __osf__
#include <sys/timers.h>
#endif

#ifdef __sun__
#include <sys/time.h>
#endif

#include <errno.h>


Alarm::Alarm(sys_time& w)
: when(w), events()
{
  ;
}

Alarm::~Alarm()
{
  ;
}

int Alarm::AttachEvent(Event& E)
{
  events.AddHead(E);
  return 0;
}

List<Event>& Alarm::GetEventList()
{
  return events;
}

int Alarm::ArmTimer(Alarm& A)
{
  sys_time  now = GetActualSystemTime();

  if(A.when > now)
    now = A.when - now;
  else
    {
      cerr << "ALARM: error on set timer: alarm time is in the past\n";
      cerr << "Alarm time: " << A.when << '\n';
      cerr << "now:        " << now << '\n';      
      return -1;
      }
    
  itimerval a;
  a.it_interval.tv_sec  = 0;
  a.it_interval.tv_usec = 0;
  a.it_value = (timeval) now;

  //cerr << "ALARM: Arming timer\n";
  //cerr << "Alarm time: " << A.when << '\n';
  //cerr << "now:        " << now << '\n';
  int err = setitimer(ITIMER_REAL,&a,NULL);
  //cerr << "ALARM: retset = " << err << '\n';
 
  if(err < 0)
    {
      cerr << "ALARM: error on set timer; errno = " << errno << '\n';
      cerr << "a.it_value.tv_sec  = " << a.it_value.tv_sec << '\n';
      cerr << "a.it_value.tv_usec = " << a.it_value.tv_usec << '\n';
      return -1;
   }
  else
    {
      return 0;
    }
}








Alarm_Manager::Alarm_Manager()
  : pending_alarm(NULL)
{
  ;
}

Alarm_Manager::~Alarm_Manager()
{
  ;
}


Alarm* Alarm_Manager::ActiveAlarm()
{
  return pending_alarm;
}


int Alarm_Manager::ScheduleNextAlarm()
{
  //cout << "ALARM MANAGER: schedule next start\n";

  Alarm* pNext = NULL;
  if(alarms_queue.Length() != 0)
    {
      alarms_queue.Out();
      if(alarms_queue.Length() != 0)
	{
	  while(alarms_queue.Length() != 0)
	    {
	      //cout << "ALARM MANAGER: schedule next\n";
	      pNext = alarms_queue.First();     
	      int err = Alarm::ArmTimer(*pNext);
	      if(err == 0)
		{
		  //cerr << "ALARM MANAGER: next alarm is scheduled\n";
		  pending_alarm = pNext;
		  return 0; 
		}
	      else
		{
		  alarms_queue.Out();
		  bad_alarms.AddTail(*pNext);
		}
	    }
	  cerr << "ALARM MANAGER: no next alarm schedulable\n";
	  pending_alarm = NULL;
	  return -1;
	}
      else
	{
	  //cerr << "ALARM MANAGER: schedule next: no more alarms\n";
	  pending_alarm = NULL;
	  return 0;
	}
    }
  else
    {
      cerr << "ALARM MANAGER: schedule next: something strange\n";
      pending_alarm = NULL;
      return -1;
    }
}


int Alarm_Manager::ScheduleNewEvent(sys_time& when,Event& E)
{
  /*
  if(!IsValidTime(when))
    return -1;
    */

  Alarm* pA = NULL;

  //cout << "ALARM MANAGER: ScheduleNewEvent when = " << when << '\n';
  if((pA = alarms_queue[when]) == NULL)
    {
      //cout << "ALARM MANAGER: when not in queue\n";
      pA = new Alarm(when);
      alarms_queue.In(when,*pA);
    }
  //else
  //cout << "ALARM MANAGER: when in queue\n";
  
  pA->AttachEvent(E);
  
  if(pending_alarm != alarms_queue.First())
    {
      pending_alarm = alarms_queue.First();
      if(Alarm::ArmTimer(*pending_alarm) == 0)
	return 0;
      else
	{
	  bad_alarms.AddTail(pending_alarm);
	  return this->ScheduleNextAlarm();
	}
    }
}

Alarm* Alarm_Manager::GetBad()
{
  if(bad_alarms.Length() == 0)
    return NULL;
  else
    return bad_alarms.RemHead();
}

Alarm* Alarm_Manager::GetLastBad()
{
  if(bad_alarms.Length() == 0)
    return NULL;
  else
    return bad_alarms.RemTail();
}

Alarm_Manager  The_Alarm_Manager;




